#ifndef _DEQUE_H_
#define _DEQUE_H_


#include "definitions.h"
#include "stack_iterator.h"
#include "linked_list.h"


namespace SCE {

/*
 * Deque class.
 */
template <class T, bool Ordered>
class Deque
{

public:
	// define and export the type Iterator
        typedef StackIterator<Deque<T, Ordered>, T> Iterator;

	// contructor
	Deque();

	// destructor
	~Deque() {  }

	// push an element into the deque 
	int push(T const &);

	// pop an element off the deque
	int pop(T &, POP_TYPE const = THRU_TAIL);

	// peek at the top of the deque
	T const & peek(PEEK_TYPE const = THRU_TAIL);

	// the linked list that holds the elements in the stack
	int isEmpty() const { return elements.isEmpty(); } 

	// the linked list that holds the elements in the stack
	int size() { return elements.size(); } 

	// the linked list that holds the elements in the stack
        Iterator const & getIterator() const { return iterator; }

private:
	// the linked list that holds the elements in the stack
	LinkedList<T, Ordered, true> elements;

	// an alias to the type Iterator in the linked list
	typedef typename LinkedList<T, Ordered, true>::Iterator ElementsIterator;
	// the iterator that will be exported	
	Iterator iterator;
};


// constructor
template <class T, bool Ordered>
Deque<T, Ordered>::Deque()
{
	iterator.setContainer(this);
}

 // push an element onto the deque 
template <class T, bool Ordered>
int Deque<T, Ordered>::push(T const & new_item)
{
	ElementsIterator elements_iterator = elements.last();
	elements_iterator = new_item;

	return 0;
}

// pop an element off the deque
template <class T, bool Ordered> 
int Deque<T, Ordered>::pop(T & element, POP_TYPE const pop_type) 
{
	ElementsIterator elements_iterator;

	if (pop_type == THRU_HEAD)
	{
		elements_iterator = elements.first();
		element = *elements_iterator;
	}
	else if (pop_type == THRU_TAIL)
	{
		elements_iterator = elements.last();
		element = *elements_iterator;
	}
	elements_iterator.remove();

	return 0;
}

// take a peek at the top of the deque 
template <class T, bool Ordered> 
T const & Deque<T, Ordered>::peek(PEEK_TYPE const peek_type)
{
	ElementsIterator elements_iterator;

	if (peek_type == THRU_HEAD)
	{
		elements_iterator = elements.first();
		return *elements_iterator;
	}
	else if (peek_type == THRU_TAIL)
	{
		elements_iterator = elements.last();
		return *elements_iterator;
	}

}

}  // namespace SCE

#endif  // _DEQUE_H_
