#ifndef _LINKED_LIST_H_
#define _LINKED_LIST_H_


#include "if.h"
#include "single_linked_list_iterator.h"
#include "doubly_linked_list_iterator.h"


namespace SCE {


/*
 * Structure that represents a node in a single linked list.
 */
template <class T, bool doubly>
struct LIST_NODE
{
	T element;  // the element associated with this node
	LIST_NODE<T, doubly> * next;  // a pointer to the next element in the list
};

/*
 * Structure that represents a node in a doubly linked list.
 */
template <class T>
struct LIST_NODE <T, true>
{
	T element;  // the element associated with this node
	LIST_NODE<T, true> * next;  // a pointer to the next element
	LIST_NODE<T, true> * previous; // a pointer to the previous element
};

	
/*
 * LinkedList class.
 */

template <class T, bool Ordered = false, bool doubly = false>
class LinkedList
{

public:
	// define and export the type Iterator
	typedef typename IF<doubly, 
	  DoublyLinkedListIterator<LIST_NODE<T, doubly>, T, Ordered>,
	  SingleLinkedListIterator<LIST_NODE<T, doubly>, T, Ordered>
	>::RET
		
	Iterator;


	// contructor
	LinkedList()
	{
		// initializing head_node
		head_node.next = NULL;

		// initializing iterator
		iterator = Iterator(&head_node);
	}

	// destructor
	~LinkedList()
	{
		if (!isEmpty())
		{
			// remove all element from the linke list
			iterator = first();

			while(iterator.hasNext())
			{
				iterator.remove();
			}
			iterator.remove();
		}
	}

	// returns true if the list is empty
	bool isEmpty() const
	{
		return head_node.next == NULL;
	}

	// returns the number of elements in the linked list
	int size()
	{
		int noof_elements = 0;
		LIST_NODE<T, doubly> * tmp = &head_node;

		while (tmp->next != NULL)
		{
			noof_elements++;
			tmp = tmp->next;
		}

                return noof_elements;
	}

	// returns an iterator to the first element of the list (or to head_node
	// if the list is empty)
	Iterator const & first()
	{
		// initialize iterator
		iterator = getIterator();

		// make it point to the first element in the list (head_node if
		// the list is empty)
		iterator++;

		// return a reference to iterator
		return iterator;
	}

	// returns an iterator to the last element of the list (or to head_node
	// if the list is empty)
	Iterator const & last()
	{
		// initialize iterator
		iterator = first();

		// run to the last element in the list 
		while(iterator.hasNext())
		{
			iterator++;
		}
	
		// return a reference to iterator
		return iterator;
	}

	// returns an iterator to the head of the list
	Iterator const & getIterator()
	{
		// initialize iterator
		iterator = Iterator(&head_node);
		// return a reference to iterator
		return iterator;
	}

private:
	// the head node of the lnked ist
	LIST_NODE<T, doubly> head_node;

        // the iterator that will be exported
	Iterator iterator;
};

}  // namespace SCE

#endif  /* _LINKED_LIST_H_ */
