#ifndef _STACK_H_
#define _STACK_H_


#include "definitions.h"
#include "stack_iterator.h"
#include "linked_list.h"


namespace SCE {


/*
 * Stack class.
 */
template <class T, bool ordered>
class Stack
{

public:
        // define and export the type Iterator
        typedef StackIterator<Stack<T, ordered>, T> Iterator;

	// contructor
	Stack(STACK_TYPE const = FILO);

	// destructor
	~Stack();

	// push an element into the stack
	int push(T const &);

	// pop an element off the stack
	int pop(T &);

	// peek at the top element in the stack
	T const & peek();

	// returns true if the stack is currently empty
	int isEmpty() const { return elements.isEmpty(); }

	// return the number of elements in the stack
	int size() { return elements.size(); }

	// returns an iterator to the elements in the stack 
        Iterator const & getIterator() const { return iterator; }

private:
	// the linked list that holds the elements in the stack
	LinkedList<T, ordered, false> elements;

	// an alias to the type Iterator in the linked list
	typedef typename LinkedList<T, ordered, false>::Iterator ElementsIterator;
	// the iterator that will be exported
	Iterator iterator;

	// the stack type (FIFO, FILO, ...)
	STACK_TYPE stack_type;
};

// constructor
template <class T, bool ordered>
Stack<T, ordered>::Stack(STACK_TYPE const _stack_type)
{
	stack_type = _stack_type;
	iterator.setContainer(this);
}

// destructor
template <class T, bool ordered>
Stack<T, ordered>::~Stack()
{
}

 // push an element onto the stack 
template <class T, bool ordered>
int Stack<T, ordered>::push(T const & new_item)
{
	ElementsIterator elements_iterator;
	if (stack_type == FIFO)
	{
		elements_iterator = elements.getIterator();
		elements_iterator = new_item;
	}
	else if (stack_type == FILO)
	{
		elements_iterator = elements.last();
		elements_iterator = new_item;
	}

	return 0;
}

// pop an element off the stack
template <class T, bool ordered> 
int Stack<T, ordered>::pop(T & element) 
{
	ElementsIterator elements_iterator = elements.last();
	element = *elements_iterator;
	elements_iterator.remove();

	return 0;
}

// take a peek at the top of the stack 
template <class T, bool ordered> 
T const & Stack<T, ordered>::peek()
{
	ElementsIterator elements_iterator = elements.last();
	return *elements_iterator;
}

}  // namespace SCE

#endif  // _STACK_H_
